import { isBoolean, isString, isFunction, includes } from '@ntks/toolbox';
import { getControl } from './component';
const moduleMap = new Map();
function convertAsyncFunctionMapToServerActionMap(funcMap) {
    const actionMap = {};
    Object.keys(funcMap).forEach(funcName => {
        const funcOrAction = funcMap[funcName];
        actionMap[funcName] = isFunction(funcOrAction)
            ? {
                name: funcName,
                type: 'server',
                execute: funcOrAction,
            }
            : funcOrAction;
    });
    return actionMap;
}
function ensureModuleExists(name) {
    if (!moduleMap.has(name)) {
        moduleMap.set(name, {
            model: undefined,
            actions: {},
            views: {},
            imports: [],
            exports: {},
            dependencies: {},
            componentRefs: {},
            components: {},
        });
    }
    return moduleMap.get(name);
}
function registerModule({ name, model = undefined, actions = {}, views = {}, imports = [], exports = {}, components = {}, }) {
    moduleMap.set(name, {
        model,
        actions: convertAsyncFunctionMapToServerActionMap(actions),
        views,
        imports,
        exports,
        dependencies: {},
        componentRefs: components,
        components: ensureModuleExists(name).components,
    });
}
function getDependencyByRef(ref) {
    const [moduleName, resourceType, resourceName] = ref.split('.');
    const module = ensureModuleExists(moduleName);
    return module.exports[resourceType] && module.exports[resourceType][resourceName];
}
function resolveDependencies() {
    moduleMap.forEach((module, name) => moduleMap.set(name, Object.assign(Object.assign({}, module), { dependencies: module.imports.reduce((prev, ref) => (Object.assign(Object.assign({}, prev), { [ref]: getDependencyByRef(ref) })), {}) })));
}
function getDependencies(moduleName, refPath) {
    const module = ensureModuleExists(moduleName);
    const dependencies = {};
    module.imports.forEach(ref => {
        const [dependencyModule, resourceType, resourceName] = ref.split('.');
        if (!dependencies[dependencyModule]) {
            dependencies[dependencyModule] = {};
        }
        if (!dependencies[dependencyModule][resourceType]) {
            dependencies[dependencyModule][resourceType] = {};
        }
        dependencies[dependencyModule][resourceType][resourceName] = module.dependencies[ref];
    });
    if (!refPath || !isString(refPath)) {
        return dependencies;
    }
    const [refModule, refResourceType] = refPath.split('.').slice(0, 2);
    const dependency = dependencies[refModule];
    if (!dependency) {
        return;
    }
    return refResourceType ? dependency[refResourceType] : dependency;
}
function resolveComponents() {
    moduleMap.forEach((module, name) => {
        const refs = module.componentRefs;
        let dependencies;
        Object.keys(refs).forEach(id => {
            const ref = refs[id];
            const useIdDirectly = isBoolean(ref);
            const refParts = isString(ref) ? ref.split('.') : [];
            let resolvedComponent;
            if (useIdDirectly || refParts.length === 1) {
                resolvedComponent = getControl(useIdDirectly ? id : ref);
            }
            else if (refParts.length === 3) {
                if (!dependencies) {
                    dependencies = getDependencies(name);
                }
                const [refModule, _, widgetName] = refParts;
                const { widgets } = dependencies[refModule];
                resolvedComponent = widgets && widgets[widgetName];
            }
            if (resolvedComponent) {
                module.components[id] = resolvedComponent;
            }
        });
    });
}
function registerModules(descriptors) {
    descriptors.forEach(registerModule);
    resolveDependencies();
    resolveComponents();
}
function getActions(moduleName) {
    return ensureModuleExists(moduleName).actions;
}
function getViews(moduleName) {
    return ensureModuleExists(moduleName).views;
}
function getComponents(moduleName, newComponents = {}) {
    const { components } = ensureModuleExists(moduleName);
    Object.keys(newComponents).forEach(name => (components[name] = newComponents[name]));
    return components;
}
function isWidgetDependency(moduleName, widgetName) {
    const module = moduleMap.get(moduleName);
    if (!module || (module.imports || []).length === 0 || !module.componentRefs[widgetName]) {
        return false;
    }
    return includes(module.componentRefs[widgetName], module.imports);
}
export { convertAsyncFunctionMapToServerActionMap, ensureModuleExists, registerModules, getActions, getViews, getDependencies, getComponents, isWidgetDependency, };
